

namespace Esb.Math
{

	/// <summary>
	/// <para>Math Comparison Routines for EsbPCS.</para>
	/// <para>Equality, inequality, greater than and less than comparisons should
	/// not be done with the operators, rather use these methods where the 
	/// precision of the comparison can be controlled.</para>
	/// <para>Methods for double, float and decimal included.</para>
	/// </summary>
	public class CompareOps
	{

		/// <summary>
		/// Dummy Constructor - Class only contains static Mathods
		/// </summary>
		private CompareOps()
		{
		} // <end of function> CompareOps()																						    {6c2f1913-51d4-4f3f-8ecb-f0c1d4897775}

		#region Comparison Constants

		/// <summary>
		/// Default Precision when doing Floating Point Comparisons of type Float (1.0e-5f).
		/// </summary>
		public const float FloatPrecisionDef = 1.0e-5f;

		/// <summary>
		/// Default Precision when doing Floating Point Comparisons of type Double (1.0e-12).
		/// </summary>
		public const double DoublePrecisionDef = 1.0e-12;

		/// <summary>
		/// Default Precision when doing Floating Point Comparisons of type Decimal (1.0e-23m).
		/// </summary>
		public const decimal DecimalPrecisionDef = 1.0e-23m;

		#endregion //Comparison Constants

		#region GetPrecision Methods

		/// <overloads>
		/// <para>Returns the Precision Value to use using the Default Precision.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Precision Value to use using the Default Precision.
		/// See <see cref="DoublePrecisionDef"/>.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		public static double GetPrecision (double x, double y)
		{
			if ((x == 0.0) || (y == 0.0))
			{
				return DoublePrecisionDef;
			} // <end of> if ((x == 0.0) || (y == 0.0))																				    {4db40ed3-5ec5-4c11-a4c7-d6f0fc154ae8}
			else
			{
				return (System.Math.Min (System.Math.Abs (x), System.Math.Abs (y)) * DoublePrecisionDef);
			} // <end of else> if ((x == 0.0) || (y == 0.0))																			    {82fdfced-82d0-4205-a406-9f8710d96527}
		} // <end of function> double GetPrecision (double x, double y)																	    {a0c8a2d8-4abc-4874-ac94-2ddf5d7a0e8b}

		/// <summary>
		/// Returns the Precision Value to use using the specified Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		/// <param name="precision">Precision Value to use - should be small and positive like <see cref="DoublePrecisionDef"/>.</param>
		public static double GetPrecision (double x, double y, double precision)
		{
			if ((x == 0.0) || (y == 0.0))
			{
				return precision;
			} // <end of> if ((x == 0.0) || (y == 0.0))																				    {ce5214d8-2ba6-42a8-95d1-3c586059d430}
			else
			{
				return (System.Math.Min (System.Math.Abs (x), System.Math.Abs (y)) * precision);
			} // <end of else> if ((x == 0.0) || (y == 0.0))																			    {312559a1-b4b3-4ae9-9f22-f4a23d421b47}
		} // <end of function> double GetPrecision (double x, double y, double...															    {7a66779a-9e63-4291-942b-d92a40b77b60}

		/// <summary>
		/// Returns the Precision Value to use using the Default Precision.
		/// See <see cref="FloatPrecisionDef"/>.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		public static float GetPrecision (float x, float y)
		{
			if ((x == 0.0f) || (y == 0.0f))
			{
				return FloatPrecisionDef;
			} // <end of> if ((x == 0.0f) || (y == 0.0f))																			    {2860c241-9131-4fe2-a04f-274d4d7a5a21}
			else
			{
				return (System.Math.Min (System.Math.Abs (x), System.Math.Abs (y)) * FloatPrecisionDef);
			} // <end of else> if ((x == 0.0f) || (y == 0.0f))																		    {48835a84-be22-4335-9b47-df67a152a314}
		} // <end of function> float GetPrecision (float x, float y)																	    {fe516e6b-f7a6-4e47-ad75-65e76b628e36}

		/// <summary>
		/// Returns the Precision Value to use using the specified Precision
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		/// <param name="precision">Precision Value to use - should be small and positive like <see cref="FloatPrecisionDef"/>.</param>
		public static float GetPrecision (float x, float y, float precision)
		{
			if ((x == 0.0f) || (y == 0.0f))
			{
				return precision;
			} // <end of> if ((x == 0.0f) || (y == 0.0f))																			    {a82e4887-d45d-4ab5-8afd-00fb3882d5bc}
			else
			{
				return (System.Math.Min (System.Math.Abs (x), System.Math.Abs (y)) * precision);
			} // <end of else> if ((x == 0.0f) || (y == 0.0f))																		    {2dbcb2fb-380e-4a22-be60-f1917d50cd71}
		} // <end of function> float GetPrecision (float x, float y, float pre...															    {43e647cb-150d-4f42-b695-96b433bb307f}

		/// <summary>
		/// Returns the Precision Value to use using the Default Precision.
		/// See <see cref="DecimalPrecisionDef"/>.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		public static decimal GetPrecision (decimal x, decimal y)
		{
			if ((x == 0.0m) || (y == 0.0m))
			{
				return DecimalPrecisionDef;
			} // <end of> if ((x == 0.0m) || (y == 0.0m))																			    {f25026e2-753c-4618-a679-a5270ed36bd0}
			else
			{
				return (System.Math.Min (System.Math.Abs (x), System.Math.Abs (y)) * DecimalPrecisionDef);
			} // <end of else> if ((x == 0.0m) || (y == 0.0m))																		    {7a9d1af2-e87b-4e81-b8b7-c225b4be6f6c}
		} // <end of function> decimal GetPrecision (decimal x, decimal y)																    {9edb097c-b13b-40d3-bc6a-ae06e5986e4a}

		/// <summary>
		/// Returns the Precision Value to use using the specified Precision
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		/// <param name="precision">Precision Value to use - should be small and positive like <see cref="DecimalPrecisionDef"/>.</param>
		public static decimal GetPrecision (decimal x, decimal y, decimal precision)
		{
			if ((x == 0.0m) || (y == 0.0m))
			{
				return precision;
			} // <end of> if ((x == 0.0m) || (y == 0.0m))																			    {55f3c569-83a8-4c92-85e6-956855e199b5}
			else
			{
				return (System.Math.Min (System.Math.Abs (x), System.Math.Abs (y)) * precision);
			} // <end of else> if ((x == 0.0m) || (y == 0.0m))																		    {0046b2d1-4c6f-44ed-9e30-669be6d7f68a}
		} // <end of function> decimal GetPrecision (decimal x, decimal y, dec...															    {d95c15a3-880a-4a7d-add2-7b41e1d254d3}

		#endregion //GetPrecision Methods

		#region SameValue Methods

		/// <overloads>
		/// <para>Returns True if the Values are within a "small" value of each other.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns True if the Values are within a "small" value of each other.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to compare.</param>
		/// <param name="y">Second Value to compare.</param>
		/// <returns>true if values are within precision of each other.</returns>
		public static bool SameValue (double x, double y)
		{
			return (System.Math.Abs (x - y) <= GetPrecision (x, y));
		} // <end of function> bool SameValue (double x, double y)																		    {8b04cbee-aafd-4471-afea-8059c8802a12}

		/// <summary>
		/// Returns True if the Values are within a "small" value computed from the supplied Precision.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>true if values are within precision of each other.</returns>
		public static bool SameValue (double x, double y, double precision)
		{
			return (System.Math.Abs (x - y) <= GetPrecision (x, y, precision));
		} // <end of function> bool SameValue (double x, double y, double prec...															    {517e84e5-28fd-4f72-86a6-ee26c26dd8bc}

		/// <summary>
		/// Returns True if the Values are within a "small" value of each other.
		/// <see cref="FloatPrecisionDef"/> defines the precision.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to compare.</param>
		/// <param name="y">Second Value to compare.</param>
		/// <returns>true if values are within precision of each other.</returns>
		public static bool SameValue (float x, float y)
		{
			return (System.Math.Abs (x - y) <= GetPrecision (x, y));
		} // <end of function> bool SameValue (float x, float y)																		    {6cd33191-dd30-4149-98dc-697117c1f782}

		/// <summary>
		/// Returns True if the Values are within a "small" value computed from the supplied Precision.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>true if values are within precision of each other.</returns>
		public static bool SameValue (float x, float y, float precision)
		{
			return (System.Math.Abs (x - y) <= GetPrecision (x, y, precision));
		} // <end of function> bool SameValue (float x, float y, float precision)															    {f9119453-041b-4274-b4ce-aaf07d724291}

		/// <summary>
		/// Returns True if the Values are within a "small" value of each other.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue"/></para>
		/// <see cref="DecimalPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to compare.</param>
		/// <param name="y">Second Value to compare.</param>
		/// <returns>true if values are within precision of each other.</returns>
		public static bool SameValue (decimal x, decimal y)
		{
			return (System.Math.Abs (x - y) <= GetPrecision (x, y));
		} // <end of function> bool SameValue (decimal x, decimal y)																	    {7dcb8590-4052-4211-9ccd-a324ff2aef98}

		/// <summary>
		/// Returns True if the Values are within a "small" value computed from the supplied Precision.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>true if values are within precision of each other.</returns>
		public static bool SameValue (decimal x, decimal y, decimal precision)
		{
			return (System.Math.Abs (x - y) <= GetPrecision (x, y, precision));
		} // <end of function> bool SameValue (decimal x, decimal y, decimal p...															    {6195a363-2c1f-41dd-96ba-e8b44c9e0fa4}

		#endregion //SameValue Methods

		#region CompareValue Methods

		/// <overloads>
		/// <para>Returns 0 if Values are within a "small" value computed from the supplied precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (double x, double y)
		{
			if (SameValue (x, y))
			{
				return 0;
			} // <end of> if (SameValue (x, y))																					    {d2aa5430-0fa4-4066-9904-264ba119516c}
			else if (x > y)
			{
				return 1;
			} // <end of else> if (SameValue (x, y))																				    {39175f7a-9ec8-4e33-822d-9bc8d2c2a9a7}
			else
			{
				return -1;
			} // <end of else> if (SameValue (x, y))																				    {08926711-900b-4b91-bc55-b2580ad1f6fc}
		} // <end of function> int CompareValue (double x, double y)																	    {b983bd50-d652-4cb4-8e81-7226889344c2}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (double x, double y, double precision)
		{
			if (SameValue (x, y, precision))
			{
				return 0;
			} // <end of> if (SameValue (x, y, precision))																			    {a361d2de-40fc-45f4-aeca-ca8005e993b8}
			else if (x > y)
			{
				return 1;
			} // <end of else> if (SameValue (x, y, precision))																		    {cd017136-2f19-4b08-9779-0cd7176a5189}
			else
			{
				return -1;
			} // <end of else> if (SameValue (x, y, precision))																		    {a7844d0b-4856-4e77-831f-c10a9d2a9094}
		} // <end of function> int CompareValue (double x, double y, double pr...															    {fd4b7977-b33d-43e5-9e51-1df0b88327ab}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// <see cref="FloatPrecisionDef"/> defines the precision.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (float x, float y)
		{
			if (SameValue (x, y))
			{
				return 0;
			} // <end of> if (SameValue (x, y))																					    {fe72025d-62b1-4ba9-b41e-c65dc235eaf1}
			else if (x > y)
			{
				return 1;
			} // <end of else> if (SameValue (x, y))																				    {1ea9a89e-814d-4e77-b8fd-9bfba335fba3}
			else
			{
				return -1;
			} // <end of else> if (SameValue (x, y))																				    {862739d6-e3c6-46fe-bf7c-d6d732d9836d}
		} // <end of function> int CompareValue (float x, float y)																		    {6af5db66-bcb3-4d49-85c1-0752801f2824}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (float x, float y, float precision)
		{
			if (SameValue (x, y, precision))
			{
				return 0;
			} // <end of> if (SameValue (x, y, precision))																			    {eb27c8b7-7795-4b5c-85be-9db6bb36180b}
			else if (x > y)
			{
				return 1;
			} // <end of else> if (SameValue (x, y, precision))																		    {a57375a5-1da3-4482-82fc-e3fbf7f014f9}
			else
			{
				return -1;
			} // <end of else> if (SameValue (x, y, precision))																		    {bc9e1ccc-ad2d-4057-a73d-051376556abb}
		} // <end of function> int CompareValue (float x, float y, float preci...															    {a1f7e1ec-079c-421a-8c02-1eae7bb6e831}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// <see cref="DecimalPrecisionDef"/> defines the precision.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (decimal x, decimal y)
		{
			if (SameValue (x, y))
			{
				return 0;
			} // <end of> if (SameValue (x, y))																					    {baa009ab-663f-43b8-b440-8ad39972897a}
			else if (x > y)
			{
				return 1;
			} // <end of else> if (SameValue (x, y))																				    {e730e9f3-6f4a-4421-8f39-6cdaab4683f9}
			else
			{
				return -1;
			} // <end of else> if (SameValue (x, y))																				    {30e0ae23-fc44-48be-a96d-74e5f30c576c}
		} // <end of function> int CompareValue (decimal x, decimal y)																	    {05b97f8d-10e5-46bb-b3c9-2232c21416dc}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to CompareOps.</param>
		/// <param name="y">Second Value to CompareOps.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (decimal x, decimal y, decimal precision)
		{
			if (SameValue (x, y, precision))
			{
				return 0;
			} // <end of> if (SameValue (x, y, precision))																			    {97f71046-e4bd-4020-8695-665ac0936662}
			else if (x > y)
			{
				return 1;
			} // <end of else> if (SameValue (x, y, precision))																		    {7fcf47f3-0518-4e58-981b-4396c88886c1}
			else
			{
				return -1;
			} // <end of else> if (SameValue (x, y, precision))																		    {e5c8ae62-740d-4c19-bcea-268665aadf10}
		} // <end of function> int CompareValue (decimal x, decimal y, decimal...															    {1dafebaa-0798-4bd8-8e39-95f4a3927674}

		#endregion //CompareValue Methods

		#region GreaterValue Methods

		/// <overloads>
		/// <para>Returns true if Values are not within a "small" value of each other, and the First Value is greater.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is greater.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (double x, double y)
		{
			return (x - y > GetPrecision (x, y));
		} // <end of function> bool GreaterValue (double x, double y)																	    {bf87083c-6c62-44fd-b34a-6e5bcadb7f2d}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is greater.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (double x, double y, double precision)
		{
			return (x - y > GetPrecision (x, y, precision));
		} // <end of function> bool GreaterValue (double x, double y, double p...															    {19fdcda8-39d6-404d-9620-434880d52dd0}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is greater.
		/// <see cref="FloatPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (float x, float y)
		{
			return (x - y > GetPrecision (x, y));
		} // <end of function> bool GreaterValue (float x, float y)																		    {ce030501-eb25-43fa-b7cb-2db85d921542}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is greater.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (float x, float y, float precision)
		{
			return (x - y > GetPrecision (x, y, precision));
		} // <end of function> bool GreaterValue (float x, float y, float prec...															    {d74bd89c-d302-4c24-ad4a-8cfcafdf3afa}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is greater.
		/// <see cref="DecimalPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (decimal x, decimal y)
		{
			return (x - y > GetPrecision (x, y));
		} // <end of function> bool GreaterValue (decimal x, decimal y)																	    {9b76a20d-27f9-4637-b94d-0ae113ce1cc5}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is greater.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (decimal x, decimal y, decimal precision)
		{
			return (x - y > GetPrecision (x, y, precision));
		} // <end of function> bool GreaterValue (decimal x, decimal y, decima...															    {00c82738-5fc6-4def-92c2-a16de90eb887}

		#endregion //GreaterValue Methods

		#region LesserValue Methods

		/// <overloads>
		/// <para>Returns true if Values are not within a "small" value of each other, and the First Value is lesser.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is lesser.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (double x, double y)
		{
			return (x - y < -GetPrecision (x, y));
		} // <end of function> bool LesserValue (double x, double y)																	    {6937f68e-8c51-4441-958b-84e4c1f1fcf2}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is lesser.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (double x, double y, double precision)
		{
			return (x - y < -GetPrecision (x, y, precision));
		} // <end of function> bool LesserValue (double x, double y, double pr...															    {2d8535ac-3d3c-4cc6-8d3c-419c7064336f}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is lesser.
		/// <see cref="FloatPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (float x, float y)
		{
			return (x - y < -GetPrecision (x, y));
		} // <end of function> bool LesserValue (float x, float y)																		    {525b6df1-f05f-4052-b448-88d6e365564d}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is lesser.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (float x, float y, float precision)
		{
			return (x - y < -GetPrecision (x, y, precision));
		} // <end of function> bool LesserValue (float x, float y, float preci...															    {4786b8cf-8d64-4f59-8925-c7c74f3d613b}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is lesser.
		/// <see cref="DecimalPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (decimal x, decimal y)
		{
			return (x - y < -GetPrecision (x, y));
		} // <end of function> bool LesserValue (decimal x, decimal y)																	    {ffd2541b-3274-4225-9b36-93ed747d1043}

		/// <summary>
		/// Returns true if Values are not within a "small" value of each other, and the First Value is lesser.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">First Value to Compare.</param>
		/// <param name="y">Second Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (decimal x, decimal y, decimal precision)
		{
			return (x - y < -GetPrecision (x, y, precision));
		} // <end of function> bool LesserValue (decimal x, decimal y, decimal...															    {365565af-fa73-4196-aedd-e052b50f11d4}

		#endregion //LesserValue Methods

		#region ValueIsZero Methods

		/// <overloads>
		/// <para>Returns true if Value is within a "small" value of 0.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns true if Value is within a "small" value of 0.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (double x)
		{
			return (System.Math.Abs (x) <= DoublePrecisionDef);
		} // <end of function> bool ValueIsZero (double x)																			    {4bf85c88-e40b-4e05-9544-39d11a2e4612}

		/// <summary>
		/// Returns true if Value is within a "small" value of 0.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (double x, double precision)
		{
			return (System.Math.Abs (x) <= precision);
		} // <end of function> bool ValueIsZero (double x, double precision)																    {09670ec2-45a4-4827-af53-6506ad64c026}

		/// <summary>
		/// Returns true if Value is within a "small" value of 0.
		/// <see cref="FloatPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (float x)
		{
			return (System.Math.Abs (x) <= FloatPrecisionDef);
		} // <end of function> bool ValueIsZero (float x)																				    {a5f8b82f-dfe1-494c-b610-a5fcb63a21d5}

		/// <summary>
		/// Returns true if Value is within a "small" value of 0.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (float x, float precision)
		{
			return (System.Math.Abs (x) <= precision);
		} // <end of function> bool ValueIsZero (float x, float precision)																    {a99d5db5-1543-4be9-95c0-9e863537cece}

		/// <summary>
		/// Returns true if Value is within a "small" value of 0.
		/// <see cref="DecimalPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (decimal x)
		{
			return (System.Math.Abs (x) <= DecimalPrecisionDef);
		} // <end of function> bool ValueIsZero (decimal x)																			    {a8e2b703-39f0-4e21-8559-f23cbc0c5865}

		/// <summary>
		/// Returns true if Value is within a "small" value of 0.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (decimal x, decimal precision)
		{
			return (System.Math.Abs (x) <= precision);
		} // <end of function> bool ValueIsZero (decimal x, decimal precision)															    {fd03279f-11bb-411c-8402-707aa18176e0}

		#endregion //ValueIsZero Methods

		#region ValueIsPositive

		/// <overloads>
		/// <para>Returns true if Value is not within a "small" value of 0 and is Postive.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Positive.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is Positive.</returns>
		public static bool ValueIsPositive (double x)
		{
			return (x > DoublePrecisionDef);
		} // <end of function> bool ValueIsPositive (double x)																			    {8f161b3e-f51f-4d12-ad00-dd1b715e659f}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Positive.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>true if X is strictly Positive.</returns>
		public static bool ValueIsPositive (double x, double precision)
		{
			return (x > precision);
		} // <end of function> bool ValueIsPositive (double x, double precision)															    {fe25e599-e662-4b5a-8fc2-38ac2acf0268}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Positive.
		/// <see cref="FloatPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is Positive.</returns>
		public static bool ValueIsPositive (float x)
		{
			return (x > FloatPrecisionDef);
		} // <end of function> bool ValueIsPositive (float x)																			    {9a149d46-5029-4d4f-b979-9792a3b8eafc}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Positive.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>true if X is strictly Positive.</returns>
		public static bool ValueIsPositive (float x, float precision)
		{
			return (x > precision);
		} // <end of function> bool ValueIsPositive (float x, float precision)															    {6e2695ab-2dce-4a12-9447-2befeadce351}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Positive.
		/// <see cref="DecimalPrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is Positive.</returns>
		public static bool ValueIsPositive (decimal x)
		{
			return (x > DecimalPrecisionDef);
		} // <end of function> bool ValueIsPositive (decimal x)																		    {a1c40668-84cc-4eee-8401-a2f9eab54177}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Positive.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>true if X is strictly Positive.</returns>
		public static bool ValueIsPositive (decimal x, decimal precision)
		{
			return (x > precision);
		} // <end of function> bool ValueIsPositive (decimal x, decimal precis...															    {fe7fedaa-d7b0-45b5-830e-c796b114ea89}

		#endregion //ValueIsPositive

		#region ValueIsNegative Methods

		/// <overloads>
		/// <para>Returns true if Value is not within a "small" value of 0 and is Postive.</para>
		/// <para>Methods for double, float and decimal included, as well as versions
		/// that allow you to specify the Precision.</para>
		/// </overloads>
		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Negative.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is Negative.</returns>
		public static bool ValueIsNegative (double x)
		{
			return (x < -DoublePrecisionDef);
		} // <end of function> bool ValueIsNegative (double x)																			    {a090f714-4f7c-462e-b1f4-e53d9dc00b07}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Negative.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>true if X is strictly Negative.</returns>
		public static bool ValueIsNegative (double x, double precision)
		{
			return (x < -precision);
		} // <end of function> bool ValueIsNegative (double x, double precision)															    {333a33b2-0b7d-4cc7-80b4-081a65eff683}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Negative.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is Negative.</returns>
		public static bool ValueIsNegative (float x)
		{
			return (x < -FloatPrecisionDef);
		} // <end of function> bool ValueIsNegative (float x)																			    {edc75056-7afc-45b8-9d42-bba7ebda0696}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Negative.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>true if X is strictly Negative.</returns>
		public static bool ValueIsNegative (float x, float precision)
		{
			return (x < -precision);
		} // <end of function> bool ValueIsNegative (float x, float precision)															    {35f38684-c973-41f5-9d02-19e57e98eb39}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Negative.
		/// <see cref="DoublePrecisionDef"/> defines the precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specifiy the precision of the comparison.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <returns>True if Value is Negative.</returns>
		public static bool ValueIsNegative (decimal x)
		{
			return (x < -DecimalPrecisionDef);
		} // <end of function> bool ValueIsNegative (decimal x)																		    {aa85c769-3c40-47fc-8044-a38a302fbe74}

		/// <summary>
		/// Returns true if Value is not within a "small" value of 0 and is Negative.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="x">Value to Compare.</param>
		/// <param name="precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>true if X is strictly Negative.</returns>
		public static bool ValueIsNegative (decimal x, decimal precision)
		{
			return (x < -precision);
		} // <end of function> bool ValueIsNegative (decimal x, decimal precis...															    {1699bc34-7047-4670-a8b1-8684772eaff2}

		#endregion //ValueIsNegative Methods

		
	} // <end of> class CompareOps																								    {71ca06fc-f690-487d-9dac-96789972073d}
} // <end of> namespace Esb.Math																									    {5cd78078-18a6-4c16-9912-dc4aae61d4f1}



