
#if (DEBUG)
// #define SHOWCNT
#endif

using System;
using Esb.Globals;

namespace Esb.Math
{

	/// <summary>
	/// Decimal Routines that are missing from C# including Sqrt, etc.
	/// </summary>
#if (DEBUG)

	public class DecimalOps
#else
	sealed public class DecimalOps
#endif

	{

		/// <summary>
		/// Dummy Constructor - Class only contains static Methods
		/// </summary>
		private DecimalOps()
		{
		}

		#region SignXY Methods

		/// <overloads>
		/// <para>Returns the FORTRAN type SIGN of the Values - basically it returns a value
		/// with the Magnitude of the First Value and the Sign of the Second Value.</para>
		/// <para>Methods for decimal, double and int parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the FORTRAN type SIGN of the Values - basically it returns a value
		/// with the Magnitude of First Value and the Sign of the Second Value.
		/// </summary>
		/// <remarks>
		/// If y is negative then Returns -Abs(x)
		/// else Returns Abs(x)
		/// </remarks>
		/// <param name="x">Value whose magnitude to process.</param>
		/// <param name="y">Value whose sign to process.</param>
		/// <returns>magnitude of x with sign of y.</returns>
		public static decimal SignXY (decimal x, decimal y)
		{
			decimal dX = System.Math.Abs (x);
			return (y < 0 ? -dX  : dX);
		}

		/// <summary>
		/// Returns the FORTRAN type SIGN of the Values - basically it returns a value
		/// with the Magnitude of First Value and the Sign of the Second Value.
		/// </summary>
		/// <remarks>
		/// If y is negative then Returns -Abs(x)
		/// else Returns Abs(x)
		/// </remarks>
		/// <param name="x">Value whose magnitude to process.</param>
		/// <param name="y">Value whose sign to process.</param>
		/// <returns>magnitude of x with sign of y.</returns>
		public static decimal SignXY (double x, double y)
		{
			decimal dX = (decimal) System.Math.Abs (x);
			return (y < 0 ? -dX  : dX);
		}

		/// <summary>
		/// Returns the FORTRAN type SIGN of the Values - basically it returns a value
		/// with the Magnitude of First Value and the Sign of the Second Value.
		/// </summary>
		/// <remarks>
		/// If y is negative then Returns -Abs(x)
		/// else Returns Abs(x)
		/// </remarks>
		/// <param name="x">Value whose magnitude to process.</param>
		/// <param name="y">Value whose sign to process.</param>
		/// <returns>magnitude of x with sign of y.</returns>
		public static decimal SignXY (int x, int y)
		{
			decimal dX = (decimal) System.Math.Abs (x);
			return (y < 0 ? -dX  : dX);
		}

		#endregion //SignXY Methods

		#region Ceiling Methods

		/// <overloads>
		/// <para>Returns the Integral Value that is greater than or equal to the specified number.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Integral Value that is greater than or equal to the specified number.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral Value that is greater than or equal to the specified number.</returns>
		public static decimal Ceiling (decimal x)
		{
			decimal Result = System.Decimal.Floor (x);

			if (!CompareOps.SameValue (x, Result))
			{
				Result++;
			}
			return Result;
		}

		/// <summary>
		/// Returns the Integral Value that is greater than or equal to the specified number.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral Value that is greater than or equal to the specified number.</returns>
		public static decimal Ceiling (double x)
		{
			decimal dX = (decimal) x;
			return Ceiling (dX);
		}

		#endregion //Ceiling Methods

		#region IntPart Methods

		/// <overloads>
		/// <para>Returns the Integral portion of the value supplied.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static decimal IntPart (decimal x)
		{

			if (x >= 0)
			{
				return System.Decimal.Floor (x);
			}
			else
			{
				return Ceiling (x);
			}
		}

		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static decimal IntPart (double x)
		{
			decimal dX = (decimal) x; 
			return IntPart (dX);
		}

		#endregion //IntPart Methods

		#region FractionPart Methods

		/// <overloads>
		/// <para>Returns the Fractional portion of the value supplied.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Fractional portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Fractional portion of the value supplied</returns>
		public static decimal FractionPart (decimal x)
		{
			return x - IntPart (x);
		}

		/// <summary>
		/// Returns the Fractional portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Fractional portion of the value supplied</returns>
		public static decimal FractionPart (double x)
		{
			decimal dX = (decimal) x;
			return FractionPart (dX);
		}

		#endregion //FractionPart Methods

		#region Powers, Roots and Logarithms

		#region Sqrt Methods

		/// <overloads>
		/// <para>Computes the Decimal Square Root using Newton's Method to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Decimal Square Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed.
		/// </exception>
		/// <param name="x">We desire the square root of this Value. Must be &gt;= 0</param>
		/// <returns>Square Root of given Value</returns>
		
		public static decimal Sqrt (decimal x)
		{

			#region Arguments Check

			if (CompareOps.ValueIsNegative (x))
			{
				throw (new ArithmeticException());
			}

			if (CompareOps.ValueIsZero (x))
			{
				return 0.0m;
			}

			#endregion

			const int MaxIterations = 100;
			decimal Guess, NewGuess;
			int Count;
			bool Done;
			Guess = 1.0m;
			Count = 0;

			Done = false;

			do
			{
				NewGuess = (x / Guess + Guess) / 2.0m;
				Count ++;

				if (Guess == NewGuess)
				{

					Done = true;
				}
				else
				{
					Guess = NewGuess;
				}
			}

			while ((!Done) && (Count < MaxIterations)); 

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Sqrt Cnt = " + Count.ToString());
#endif

			return Guess;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Decimal Square Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed.
		/// </exception>
		/// <param name="x">We desire the square root of this Value. Must be &gt;= 0</param>
		/// <returns>Square Root of given Value</returns>
		public static decimal Sqrt (double x)
		{
			decimal dValue = (decimal) x;
			return Sqrt (dValue);
		}

		/// <summary>
		/// Computes the Decimal Square Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed.
		/// </exception>
		/// <param name="x">We desire the square root of this Value. Must be &gt;= 0</param>
		/// <returns>Square Root of given Value</returns>
		public static decimal Sqrt (int x)
		{
			decimal dValue = (decimal) x;
			return Sqrt (dValue);
		}

		#endregion //Sqrt Methods

		#region IntPow Methods

		/// <overloads>
		/// <para>Computes the value of a Decimal raised to an Integer Power.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the value of a Decimal raised to an Integer Power.
		/// </summary>
		/// <param name="x">The Value to be raised to the specified power.</param>
		/// <param name="y">The Power to raise the specified value to.</param>
		/// <returns>x ^ y, ie x raised to the power of y.</returns>
		public static decimal IntPow (decimal x, int y)
		{

			#region Arguments Check

			if ((CompareOps.ValueIsZero (x))||(CompareOps.SameValue (x, 1.0m)))
			{
				return x;
			}

			if (CompareOps.SameValue (x, -1.0m))
			{

				if (IntOps.Odd (y))
				{
					return -1.0m;
				}
				else
				{
					return 1.0m;
				}
			}

			if (y == 0)
			{
				return 1m;
			}

			#endregion

			decimal Result = 1.0m;

			for (int I = 1; I <= System.Math.Abs (y); I++)
			{
				Result = Result * x;
			}

			if (y < 0)
			{
				return 1.0m / Result;
			}
			else
			{
				return Result;
			}
		}

		/// <summary>
		/// Computes the value of a Decimal raised to an Integer Power.
		/// </summary>
		/// <param name="x">The Value to be raised to the specified power.</param>
		/// <param name="y">The Power to raise the specified value to.</param>
		/// <returns>x ^ y, ie x raised to the power of y.</returns>
		public static decimal IntPow (double x, int y)
		{
			decimal dX = (decimal) x;
			return IntPow (dX, y);
		}

		/// <summary>
		/// Computes the value of a Decimal raised to an Integer Power.
		/// </summary>
		/// <param name="x">The Value to be raised to the specified power.</param>
		/// <param name="y">The Power to raise the specified value to.</param>
		/// <returns>x ^ y, ie x raised to the power of y.</returns>
		public static decimal IntPow (int x, int y)
		{
			decimal dX = (decimal) x;
			return IntPow (dX, y);
		}

		#endregion //IntPow Methods

		#region NthRoot Methods

		/// <overloads>
		/// <para>Computes the Decimal Nth Root using Newton's Method to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Decimal Nth Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed or n &lt; 2.
		/// </exception>
		/// <param name="x">We desire the Nth root of this Value. Must be &gt;= 0</param>
		/// <param name="n">The Level of Root - ie Nth Root.</param>
		/// <returns>Nth Root of given Value</returns>
		public static decimal NthRoot (decimal x, int n)
		{

			#region Arguments Check

			if ((CompareOps.ValueIsNegative (x)) || (n < 2))
			{
				throw (new ArithmeticException());
			}

			if (CompareOps.ValueIsZero (x))
			{
				return 0.0m;
			}

			#endregion

			const int MaxIterations = 100;
			decimal Guess, NewGuess;
			int Count;
			bool Done;
			Guess = 1;
			Count = 0;

			Done = false;

			do
			{
				NewGuess = (x / IntPow (Guess, n - 1) + (n - 1) * Guess) / n;
				Count ++;

				if (Guess == NewGuess)
				{

					Done = true;
				}
				else
				{
					Guess = NewGuess;
				}
			}

			while ((!Done) && (Count < MaxIterations)); 

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("NthRoot Cnt = " + Count.ToString());
#endif

			return Guess;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Decimal Nth Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed or n &lt; 2.
		/// </exception>
		/// <param name="x">We desire the Nth root of this Value. Must be &gt;= 0</param>
		/// <param name="n">The Level of Root - ie Nth Root.</param>
		/// <returns>Nth Root of given Value</returns>
		public static decimal NthRoot (double x, int n)
		{
			decimal dValue = (decimal) x;
			return NthRoot (dValue, n);
		}

		/// <summary>
		/// Computes the Decimal Nth Root using Newton's Method.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a negative value is passed or n &lt; 2.
		/// </exception>
		/// <param name="x">We desire the Nth root of this Value. Must be &gt;= 0</param>
		/// <param name="n">The Level of Root - ie Nth Root.</param>
		public static decimal NthRoot (int x, int n)
		{
			decimal dValue = (decimal) x;
			return NthRoot (dValue, n);
		}

		#endregion //NthRoot Methods

		#region Log Methods

		/// <overloads>
		/// <para>Computes the Natural Logarithm using Series Expansion to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Natural Logarithm using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the natural logarithm of this Value. Must be &gt; 0</param>
		/// <returns>Natural Logarithm of given Value</returns>
		public static decimal Log (decimal x)
		{

			#region Arguments Check

			if (!CompareOps.ValueIsPositive (x))
			{
				throw (new ArithmeticException());
			}

			if (CompareOps.SameValue (x, 1.0m))
			{
				return 0.0m;
			}

			#endregion

			const int MaxIterations = 50000;
			decimal Y = (x - 1) / (x + 1);
			decimal SqrY = Y * Y;
			bool Done = false;
			decimal Result = Y;
			decimal OldResult = Result;
			decimal Z = Y;
			int Cnt = 1;

			do
			{
				Cnt = Cnt + 2;
				Z = Z * SqrY;
				Result = Result + Z / Cnt;

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaxIterations));

			#region count

#if (SHOWCNT)
			System.Console.WriteLine ("Log Cnt = " + Cnt.ToString());
#endif

			return 2 * Result;

			#endregion //count

		}

		/// <summary>
		/// Computes the Natural Logarithm using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the natural logarithm of this Value. Must be &gt; 0</param>
		/// <returns>Natural Logarithm of given Value</returns>
		public static decimal Log (double x)
		{
			decimal dX = (decimal) x;
			return Log (dX);
		}

		/// <summary>
		/// Computes the Natural Logarithm using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the natural logarithm of this Value. Must be &gt; 0</param>
		/// <returns>Natural Logarithm of given Value</returns>
		public static decimal Log (int x)
		{
			decimal dX = (decimal) x;
			return Log (dX);
		}

		#endregion //Log Methods

		#region Log10 Methods

		/// <overloads>
		/// <para>Computes the Logarithm to Base 10 using Series Expansion to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Logarithm to Base 10 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the logarithm to base 10 of this Value. Must be &gt; 0</param>
		/// <returns>Logaritm to base 10 of given Value</returns>
		public static decimal Log10 (decimal x)
		{
			return Log (x) / DecimalGlobals.LN10;
		}

		/// <summary>
		/// Computes the Logarithm to Base 10 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the logarithm to base 10 of this Value.Must be &gt; 0</param>
		/// <returns>Logaritm to base 10 of given Value</returns>
		public static decimal Log10 (double x)
		{
			decimal dX = (decimal) x;
			return Log10 (dX);
		}

		/// <summary>
		/// Computes the Logarithm to Base 10 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the logarithm to base 10 of this Value. Must be &gt; 0</param>
		/// <returns>Logaritm to base 10 of given Value</returns>
		public static decimal Log10 (int x)
		{
			decimal dX = (decimal) x;
			return Log10 (dX);
		}

		#endregion //Log10 Methods

		#region Log2 Methods

		/// <overloads>
		/// <para>Computes the Logarithm to Base 2 using Series Expansion to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Logarithm to Base 2 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the logarithm to base 2 of this Value. Must be &gt; 0</param>
		/// <returns>Logaritm to base 2 of given Value</returns>
		public static decimal Log2 (decimal x)
		{
			return Log (x) / DecimalGlobals.LN2;
		}

		/// <summary>
		/// Computes the Logarithm to Base 2 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the logarithm to base 2 of this Value. Must be &gt; 0</param>
		/// <returns>Logaritm to base 2 of given Value</returns>
		public static decimal Log2 (double x)
		{
			decimal dX = (decimal) x;
			return Log2 (dX);
		}

		/// <summary>
		/// Computes the Logarithm to Base 2 using Series Expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when a non-positive value is passed.
		/// </exception>
		/// <param name="x">We desire the logarithm to base 2 of this Value. Must be &gt; 0</param>
		/// <returns>Logaritm to base 2 of given Value</returns>
		public static decimal Log2 (int x)
		{
			decimal dX = (decimal) x;
			return Log2 (dX);
		}

		#endregion //Log2 Methods

		#region Exp Methods

		/// <overloads>
		/// <para>Computes the Exponential Function using Series Expansion to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Exponential Function using Series Expansion.
		/// </summary>
		/// <param name="x">Power to raise the Natural Constant, e, to</param>
		/// <returns>e^x, Natural Constant raised to the power x</returns>
		public static decimal Exp (decimal x)
		{

			#region Arguments Check

			if (CompareOps.ValueIsZero (x))
			{
				return 1.0m;
			}

			#endregion

			const int MaxIterations = 50000;
			decimal Y = x;
			bool Done = false;
			decimal Result = 1 + Y;
			decimal OldResult = Result;
			int Cnt = 1;

			do
			{
				Cnt++;
				Y = (Y / Cnt) * x;
				Result = Result + Y;

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaxIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Exp Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Exponential Function using Series Expansion.
		/// </summary>
		/// <param name="x">Power to raise the Natural Constant, e, to</param>
		/// <returns>e^x, Natural Constant raised to the power x</returns>
		public static decimal Exp (double x)
		{
			decimal dX = (decimal) x;
			return Exp (dX);
		}

		/// <summary>
		/// Computes the Exponential Function using Series Expansion.
		/// </summary>
		/// <param name="x">Power to raise the Natural Constant, e, to</param>
		/// <returns>e^x, Natural Constant raised to the power x</returns>
		public static decimal Exp (int x)
		{
			decimal dX = (decimal) x;
			return Exp (dX);
		}

		#endregion //Exp Methods

		#region TenPow Methods

		/// <overloads>
		/// <para>Computes the Specified Power of 10 using Series Expansion to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Specified Power of 10 using series expansion.
		/// </summary>
		/// <param name="x">Power to raise 10 to</param>
		/// <returns>10 ^ x, 10 raised to the power x</returns>
		public static decimal TenPow (decimal x)
		{
			return Exp (x * DecimalGlobals.LN10);
		}

		/// <summary>
		/// Computes the Specified Power of 10 using series expansion.
		/// </summary>
		/// <param name="x">Power to raise 10 to</param>
		/// <returns>10 ^ x, 10 raised to the power x</returns>
		public static decimal TenPow (double x)
		{
			decimal dX = (decimal) x;
			return TenPow (dX);
		}

		/// <summary>
		/// Computes the Specified Power of 10 using series expansion.
		/// </summary>
		/// <param name="x">Power to raise 10 to</param>
		/// <returns>10 ^ x, 10 raised to the power x</returns>
		public static decimal TenPow (int x)
		{
			decimal dX = (decimal) x;
			return TenPow (dX);
		}

		#endregion //TenPow Methods

		#region TwoPow Methods

		/// <overloads>
		/// <para>Computes the Specified Power of 2 using Series Expansion to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Specified Power of 2 using series expansion.
		/// </summary>
		/// <param name="x">Power to raise 2 to</param>
		/// <returns>2 ^ x, 2 raised to the power x</returns>
		public static decimal TwoPow (decimal x)
		{
			return Exp (x * DecimalGlobals.LN2);
		}

		/// <summary>
		/// Computes the Specified Power of 2 using series expansion.
		/// </summary>
		/// <param name="x">Power to raise 2 to</param>
		/// <returns>2 ^ x, 2 raised to the power x</returns>
		public static decimal TwoPow (double x)
		{
			decimal dX = (decimal) x;
			return TwoPow (dX);
		}

		/// <summary>
		/// Computes the Specified Power of 2 using series expansion.
		/// </summary>
		/// <param name="x">Power to raise 2 to</param>
		/// <returns>2 ^ x, 2 raised to the power x</returns>
		public static decimal TwoPow (int x)
		{
			decimal dX = (decimal) x;
			return TwoPow (dX);
		}

		#endregion //TwoPow Methods

		#region Pow Methods

		/// <overloads>
		/// <para>Computes the First Value raised to the power of the Second Value using Series Expansion to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (decimal x, decimal y)
		{

			#region Arguments Check

			if (CompareOps.ValueIsZero (y))
			{
				return 1.0m;
			}

			if (CompareOps.ValueIsZero (x))
			{

				if (y < 0)
				{
					throw (new ArithmeticException());
				}
				else
				{
					return 0.0m;
				}
			}

			if (CompareOps.SameValue (y, System.Decimal.Truncate (y)))
			{
				return IntPow (x, System.Decimal.ToInt32 (y));
			}

			if (x < 0)
			{
				throw (new ArithmeticException());
			}

			#endregion

			return Exp (y * Log (x));
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (double x, double y)
		{
			decimal dX = (decimal) x;
			decimal dY = (decimal) y;
			return Pow (dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (int x, int y)
		{
			decimal dX = (decimal) x;
			decimal dY = (decimal) y;
			return Pow (dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (double x, int y)
		{
			decimal dX = (decimal) x;
			decimal dY = (decimal) y;
			return Pow (dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (int x, double y)
		{
			decimal dX = (decimal) x;
			decimal dY = (decimal) y;
			return Pow (dX, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (decimal x, double y)
		{
			decimal dY = (decimal) y;
			return Pow (x, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (double x, decimal y)
		{
			decimal dX = (decimal) x;
			return Pow (dX, y);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (decimal x, int y)
		{
			decimal dY = (decimal) y;
			return Pow (x, dY);
		}

		/// <summary>
		/// Computes the First Value raised to the power of the Second Value using series expansion.
		/// </summary>
		/// <exception cref="ArithmeticException">
		/// Thrown when X is Zero and a Negative Power,
		/// or a Negative X is raised to a non-integral power.
		/// </exception>
		/// <param name="x">Value to raise</param>
		/// <param name="y">Power to use</param>
		/// <returns>x ^ y, x raised to the power y</returns>
		public static decimal Pow (int x, decimal y)
		{
			decimal dX = (decimal) x;
			return Pow (dX, y);
		}

		#endregion //Pow Methods

		#endregion //Powers, Roots and Logarithms

		#region Trigonometry

		#region Angle Conversions

		#region AdjustAngleRad Methods

		/// <overloads>
		/// <para> Adjust the Angle so that it reduced to an equivalent angle within a smaller 
		/// range of values.
		/// Works in Radians.</para>
		/// <para>Methods for decimal, double and int Parameters. Also their are overloaded
		/// versions that allow whether you want the angle reduced to a postive angle or
		/// not.</para>
		/// </overloads>
		/// <summary>
		/// Adjust the Angle so that it lies between -PI (exclusive) and PI (inclusive)
		/// if posAngle is False, or between 0 (inclusive) and 2 * PI (exclusive)
		/// if posAngle is True.
		/// Works in Radians.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <param name="posAngle">When true the Angle returned is Positive, ie
		/// between 0 and 2PI, when false then the Angle is between -PI and PI.</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad (decimal angleInRadians, bool posAngle)
		{
			decimal Result = angleInRadians;

			if (!CompareOps.LesserValue (Result, DecimalGlobals.TwoPI))
			{
				Result = Result - System.Decimal.Floor (Result / DecimalGlobals.TwoPI)
					* DecimalGlobals.TwoPI;
			}

			if (CompareOps.ValueIsNegative (Result))
			{
				Result = Result + (System.Decimal.Floor (System.Math.Abs(Result) 
					/ DecimalGlobals.TwoPI) + 1.0m) * DecimalGlobals.TwoPI;
			}

			if (posAngle || (!CompareOps.GreaterValue (Result, DecimalGlobals.PI)))
			{
				return Result;
			}
			else
			{
				return Result - DecimalGlobals.TwoPI;
			}
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -PI (exclusive) and PI (inclusive)
		/// if posAngle is False, or between 0 (inclusive) and 2 * PI (exclusive)
		/// if posAngle is True.
		/// Works in Radians.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <param name="posAngle">When true the Angle returned is Positive, ie
		/// between 0 and 2PI, when false then the Angle is between -PI and PI.</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad (double angleInRadians, bool posAngle)
		{
			decimal dX = (decimal) angleInRadians;
			return AdjustAngleRad (dX, posAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -PI (exclusive) and PI (inclusive)
		/// if posAngle is False, or between 0 (inclusive) and 2 * PI (exclusive)
		/// if posAngle is True.
		/// Works in Radians.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <param name="posAngle">When true the Angle returned is Positive, ie
		/// between 0 and 2PI, when false then the Angle is between -PI and PI.</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad (int angleInRadians, bool posAngle)
		{
			decimal dX = (decimal) angleInRadians;
			return AdjustAngleRad (dX, posAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -PI (exclusive) and PI (inclusive)
		/// Works in Radians.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad (decimal angleInRadians)
		{
			return AdjustAngleRad (angleInRadians, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -PI (exclusive) and PI (inclusive)
		/// Works in Radians.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad (double angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return AdjustAngleRad (dX, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -PI (exclusive) and PI (inclusive)
		/// Works in Radians.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Adjusted Angle in Radians</returns>
		public static decimal AdjustAngleRad (int angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return AdjustAngleRad (dX, false);
		}

		#endregion //AdjustAngleRad Methods

		#region AdjustAngleDeg Methods

		/// <overloads>
		/// <para> Adjust the Angle so that it reduced to an equivalent angle within a smaller 
		/// range of values.
		/// Works in Degrees.</para>
		/// <para>Methods for decimal, double and int Parameters. Also their are overloaded
		/// versions that allow whether you want the angle reduced to a postive angle or
		/// not.</para>
		/// </overloads>
		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// if posAngle is False, or between 0 (inclusive) and 360 (exclusive)
		/// if posAngle is True.
		/// Works in Degrees.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <param name="posAngle">When true the Angle returned is Positive, ie
		/// between 0 and 360, when false then the Angle is between -180 and 180.</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg (decimal angleInDegrees, bool posAngle)
		{
			decimal Result = angleInDegrees;

			if (!CompareOps.LesserValue (Result, 360.0m))
			{
				Result = Result - System.Decimal.Floor (Result / 360.0m) * 360.0m;
			}

			if (CompareOps.ValueIsNegative (Result))
			{
				Result = Result + (System.Decimal.Floor (System.Math.Abs (Result) / 360.0m) + 1.0m) * 360.0m;
			}

			if (posAngle || (!CompareOps.GreaterValue (Result, 180.0m)))
			{
				return Result;
			}
			else
			{
				return Result - 360.0m;
			}
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// if posAngle is False, or between 0 (inclusive) and 360 (exclusive)
		/// if posAngle is True.
		/// Works in Degrees.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <param name="posAngle">When true the Angle returned is Positive, ie
		/// between 0 and 360, when false then the Angle is between -180 and 180.</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg (double angleInDegrees, bool posAngle)
		{
			decimal dX = (decimal) angleInDegrees;
			return AdjustAngleDeg (dX, posAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// if posAngle is False, or between 0 (inclusive) and 360 (exclusive)
		/// if posAngle is True.
		/// Works in Degrees.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <param name="posAngle">When true the Angle returned is Positive, ie
		/// between 0 and 360, when false then the Angle is between -180 and 180.</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg (int angleInDegrees, bool posAngle)
		{
			decimal dX = (decimal) angleInDegrees;
			return AdjustAngleDeg (dX, posAngle);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// Works in Degrees.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg (decimal angleInDegrees)
		{
			return AdjustAngleDeg (angleInDegrees, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// Works in Degrees.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg (double angleInDegrees)
		{
			decimal dX = (decimal) angleInDegrees;
			return AdjustAngleDeg (dX, false);
		}

		/// <summary>
		/// Adjust the Angle so that it lies between -180 (exclusive) and 180 (inclusive)
		/// Works in Degrees.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Adjusted Angle in Degrees</returns>
		public static decimal AdjustAngleDeg (int angleInDegrees)
		{
			decimal dX = (decimal) angleInDegrees;
			return AdjustAngleDeg (dX, false);
		}

		#endregion //AdjustAngleDeg Methods

		#region Deg2Rad Methods

		/// <overloads>
		/// <para>Converts Degrees into Radians to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts Degrees into Radians.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Deg2Rad (decimal angleInDegrees)
		{
			return angleInDegrees * DecimalGlobals.OneDegree;
		}

		/// <summary>
		/// Converts Degrees into Radians.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Deg2Rad (double angleInDegrees)
		{
			decimal dX = (decimal) angleInDegrees;
			return dX * DecimalGlobals.OneDegree;
		}

		/// <summary>
		/// Converts Degrees into Radians.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Deg2Rad (int angleInDegrees)
		{
			decimal dX = (decimal) angleInDegrees;
			return dX * DecimalGlobals.OneDegree;
		}

		#endregion //Deg2Rad Methods

		#region Rad2Deg Methods

		/// <overloads>
		/// <para>Converts Radians into Degrees to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts Radians into Degrees.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Rad2Deg (double angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return dX * DecimalGlobals.OneRadian;
		}

		/// <summary>
		/// Converts Radians into Degrees.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Rad2Deg (int angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return dX * DecimalGlobals.OneRadian;
		}

		/// <summary>
		/// Converts Radians into Degrees.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Rad2Deg (decimal angleInRadians)
		{
			return angleInRadians * DecimalGlobals.OneRadian;
		}

		#endregion //Rad2Deg Methods

		#region Grad2Rad Methods

		/// <overloads>
		/// <para>Converts Grads into Radians to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts Grads into Radians.
		/// </summary>
		/// <param name="angleInGrads">Angle in Grads</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Grad2Rad (decimal angleInGrads)
		{
			return angleInGrads * DecimalGlobals.PI * 0.005m;
		}

		/// <summary>
		/// Converts Grads into Radians.
		/// </summary>
		/// <param name="angleInGrads">Angle in Grads</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Grad2Rad (double angleInGrads)
		{
			decimal dX = (decimal) angleInGrads; 
			return Grad2Rad (dX);
		}

		/// <summary>
		/// Converts Grads into Radians.
		/// </summary>
		/// <param name="angleInGrads">Angle in Grads</param>
		/// <returns>Angle In Radians</returns>
		public static decimal Grad2Rad (int angleInGrads)
		{
			decimal dX = (decimal) angleInGrads; 
			return Grad2Rad (dX);
		}

		#endregion //Grad2Rad Methods

		#region Rad2Grad Methods

		/// <overloads>
		/// <para>Converts Radians into Grads to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts Radians into Grads.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Rad2Grad (decimal angleInRadians)
		{
			return (200.0m * DecimalGlobals.InvPI) * angleInRadians;
		}

		/// <summary>
		/// Converts Radians into Grads.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Rad2Grad (double angleInRadians)
		{
			decimal dX = (decimal) angleInRadians; 
			return Rad2Grad (dX);
		}

		/// <summary>
		/// Converts Radians into Grads.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Rad2Grad (int angleInRadians)
		{
			decimal dX = (decimal) angleInRadians; 
			return Rad2Grad (dX);
		}

		#endregion //Rad2Grad Methods

		#region Grad2Deg Methods

		/// <overloads>
		/// <para>Converts Grads into Degrees to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts Grads into Degrees.
		/// </summary>
		/// <param name="angleInGrads">Angle in Grads</param>
		/// <returns>Angle In Degrees</returns>
		public static decimal Grad2Deg (decimal angleInGrads)
		{
			return angleInGrads * 0.9m;
		}

		/// <summary>
		/// Converts Grads into Degrees.
		/// </summary>
		/// <param name="angleInGrads">Angle in Grads</param>
		/// <returns>Angle In Degrees</returns>
		public static decimal Grad2Deg (double angleInGrads)
		{
			decimal dX = (decimal) angleInGrads; 
			return Grad2Deg (dX);
		}

		/// <summary>
		/// Converts Grads into Degrees.
		/// </summary>
		/// <param name="angleInGrads">Angle in Grads</param>
		/// <returns>Angle In Degrees</returns>
		public static decimal Grad2Deg (int angleInGrads)
		{
			decimal dX = (decimal) angleInGrads; 
			return Grad2Deg (dX);
		}

		#endregion //Grad2Deg Methods

		#region Deg2Grad Methods

		/// <overloads>
		/// <para>ConvertsDegrees into Grads to almost full accuracy.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts Degrees into Grads.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Deg2Grad (decimal angleInDegrees)
		{
			return angleInDegrees / 0.9m;
		}

		/// <summary>
		/// Converts Degrees into Grads.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Deg2Grad (double angleInDegrees)
		{
			decimal dX = (decimal) angleInDegrees; 
			return Deg2Grad (dX);
		}

		/// <summary>
		/// Converts Degrees into Grads.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Angle in Grads</returns>
		public static decimal Deg2Grad (int angleInDegrees)
		{
			decimal dX = (decimal) angleInDegrees; 
			return Deg2Grad (dX);
		}

		#endregion //Deg2Grad Methods

		#region Dms2Deg Methods

		/// <overloads>
		/// <para>Converts Degrees/Minutes/Seconds into Decimal Degrees.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts Degrees/Minutes/Seconds into Decimal Degrees. 
		/// </summary>
		/// <param name="varDegrees">Number of Degrees.</param>
		/// <param name="varMinutes">Number of Minutes.</param>
		/// <param name="varSeconds">Number of Seconds.</param>
		/// <returns> </returns>
		public static decimal Dms2Deg (decimal varDegrees, decimal varMinutes, decimal varSeconds)
		{
			return varDegrees + varMinutes / 60 + varSeconds / 3600;
		}

		/// <summary>
		/// Converts Degrees/Minutes/Seconds into Decimal Degrees. 
		/// </summary>
		/// <param name="varDegrees">Number of Degrees.</param>
		/// <param name="varMinutes">Number of Minutes.</param>
		/// <param name="varSeconds">Number of Seconds.</param>
		/// <returns> </returns>
		public static decimal Dms2Deg (double varDegrees, double varMinutes, double varSeconds)
		{
			decimal decDegrees = (decimal) varDegrees;
			decimal decMinutes = (decimal) varMinutes;
			decimal decSeconds = (decimal) varSeconds;
			return Dms2Deg (decDegrees, decMinutes, decSeconds);
		}

		/// <summary>
		/// Converts Degrees/Minutes/Seconds into Decimal Degrees. 
		/// </summary>
		/// <param name="varDegrees">Number of Degrees.</param>
		/// <param name="varMinutes">Number of Minutes.</param>
		/// <param name="varSeconds">Number of Seconds.</param>
		/// <returns> </returns>
		public static decimal Dms2Deg (int varDegrees, int varMinutes, int varSeconds)
		{
			decimal decDegrees = (decimal) varDegrees;
			decimal decMinutes = (decimal) varMinutes;
			decimal decSeconds = (decimal) varSeconds;
			return Dms2Deg (decDegrees, decMinutes, decSeconds);
		}

		#endregion //Dms2Deg Methods

		#region Deg2Dms Methods

		/// <overloads>
		/// <para>Converts a Decimal Degree into Degrees/Minutes/Seconds. Returned values are
		/// positive with Sign containing the <see cref="System.Math.Sign"/> of the original
		/// value.</para>
		/// <para>Methods for decimal and double Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Converts a Decimal Degree into Degrees/Minutes/Seconds. Returned values are
		/// positive with Sign containing the <see cref="System.Math.Sign"/> of the original
		/// value.
		/// </summary>
		/// <param name="decDegrees">Degrees in decimal format. Can be Negative.</param>
		/// <param name="varDegrees">Whole Degrees, &gt;= 0</param>
		/// <param name="varMinutes">Whole Minutes, &gt;= 0</param>
		/// <param name="varSeconds">Seconds, including fractional portion, &gt;= 0</param>
		/// <param name="varSign">1 if &gt; 0, 0 if = 0, -1 if &lt; 0</param>
		/// <returns> </returns>
		public static void Deg2Dms (decimal decDegrees, out int varDegrees, out int varMinutes,
			out decimal varSeconds, out int varSign)
		{
			varSign = System.Math.Sign (decDegrees);
			decimal decX = System.Math.Abs (decDegrees);
			varDegrees = IntOps.Floor (decX);
			decX = FractionPart (decX) * 60;
			varMinutes = IntOps.Floor (decX);
			varSeconds = FractionPart (decX) * 60;
		}

		/// <summary>
		/// Converts a Decimal Degree into Degrees/Minutes/Seconds. Returned values are
		/// positive with Sign containing the <see cref="System.Math.Sign"/> of the original
		/// value.
		/// </summary>
		/// <param name="decDegrees">Degrees in decimal format. Can be Negative.</param>
		/// <param name="varDegrees">Whole Degrees, &gt;= 0</param>
		/// <param name="varMinutes">Whole Minutes, &gt;= 0</param>
		/// <param name="varSeconds">Seconds, including fractional portion, &gt;= 0</param>
		/// <param name="varSign">1 if &gt; 0, 0 if = 0, -1 if &lt; 0</param>
		/// <returns> </returns>
		public static void Deg2Dms (double decDegrees, out int varDegrees, out int varMinutes,
			out decimal varSeconds, out int varSign)
		{
			decimal dX = (decimal) decDegrees; 
			Deg2Dms (dX, out varDegrees, out varMinutes, out varSeconds, out varSign);
		}

		#endregion //Deg2Dms Methods

		#endregion //Angle Conversions

		#region Radian Trig Ops

		#region Sin Methods

		/// <overloads>
		/// <para>Computes the Sine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </overloads>
		/// <summary>
		/// <para>Computes the Sine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal Sin (decimal angleInRadians)
		{
			decimal vAngle = AdjustAngleRad (angleInRadians);

			#region Arguments Check

			if (CompareOps.ValueIsZero (vAngle))
			{
				return 0.0m;
			}

			#endregion

			const int MaxIterations = 50000;
			decimal Y = vAngle;
			bool Done = false;
			bool Negative = false;
			decimal Result = Y;
			decimal OldResult = Result;
			int Cnt = 1;

			do
			{
				Negative = !Negative;
				Cnt = Cnt + 2;
				Y = (Y / (Cnt * (Cnt - 1))) * vAngle * vAngle;

				if (Negative)
				{
					Result = Result - Y;
				}
				else
				{
					Result = Result + Y;
				}

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaxIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Sin Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// <para>Computes the Sine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal Sin (double angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Sin (dX);
		}

		/// <summary>
		/// <para>Computes the Sine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal Sin (int angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Sin (dX);
		}

		#endregion //Sin Methods

		#region Cos Methods

		/// <overloads>
		/// <para>Computes the Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </overloads>
		/// <summary>
		/// <para>Computes the Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal Cos (decimal angleInRadians)
		{
			decimal vAngle = AdjustAngleRad (angleInRadians);

			#region Arguments Check

			if (CompareOps.ValueIsZero (vAngle))
			{
				return 1.0m;
			}

			#endregion

			const int MaxIterations = 50000;
			decimal Y = 1.0m;
			bool Done = false;
			bool Negative = false;
			decimal Result = Y;
			decimal OldResult = Result;
			int Cnt = 0;

			do
			{
				Negative = !Negative;
				Cnt = Cnt + 2;
				Y = (Y / (Cnt * (Cnt - 1))) * vAngle * vAngle;

				if (Negative)
				{
					Result = Result - Y;
				}
				else
				{
					Result = Result + Y;
				}

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaxIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Cos Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// <para>Computes the Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal Cos (double angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Cos (dX);
		}

		/// <summary>
		/// <para>Computes the Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>If you need both Sine and Cosine then <see cref="SinCos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal Cos (int angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Cos (dX);
		}

		#endregion //Cos Methods

		#region SinCos Methods

		/// <overloads>
		/// <para>Computes the Sine and Cosine of the given angle (in Radians) using Series Expansion.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Sine and Cosine of the given angle (in Radians) using Series Expansion.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <param name="sinX">Sine of the Angle</param>
		/// <param name="cosX">Cosine of the Angle</param>
		public static void SinCos (decimal angleInRadians, out decimal sinX, out decimal cosX)
		{
			decimal vAngle = AdjustAngleRad (angleInRadians);

			#region Arguments Check

			if (CompareOps.ValueIsZero (vAngle))
			{
				sinX = 0.0m;
				cosX = 1.0m;
				return;
			}

			#endregion

			const int MaxIterations = 50000;
			bool Negative = false;
			decimal Y = vAngle;
			bool Done = false;
			cosX = 1.0m;
			sinX = Y;
			decimal OldsinX = sinX;
			decimal OldcosX = cosX;
			int Cnt = 1;

			do
			{
				Cnt++;
				Negative = IntOps.Odd (Cnt / 2);
				Y = (Y / Cnt) * vAngle;

				if (Negative)
				{
					cosX = cosX - Y;
				}
				else
				{
					cosX = cosX + Y;
				}
				Cnt++;
				Y = (Y / Cnt) * vAngle;

				if (Negative)
				{
					sinX = sinX - Y;
				}
				else
				{
					sinX = sinX + Y;
				}

				if (CompareOps.SameValue (sinX, OldsinX) && CompareOps.SameValue (cosX, OldcosX))
				{

					Done = true;
				}
				else
				{
					OldsinX = sinX;
					OldcosX = cosX;
				}
			}

			while ((!Done) && (Cnt < MaxIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("SinCos Cnt = " + Cnt.ToString());
#endif

			return;

			#endregion //Count

		}

		/// <summary>
		/// Computes the Sine and Cosine of the given angle (in Radians) using Series Expansion.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <param name="sinX">Sine of the Angle</param>
		/// <param name="cosX">Cosine of the Angle</param>
		public static void SinCos (double angleInRadians, out decimal sinX, out decimal cosX)
		{
			decimal dX = (decimal) angleInRadians;
			SinCos (dX, out sinX, out cosX);
			return;
		}

		/// <summary>
		/// Computes the Sine and Cosine of the given angle (in Radians) using Series Expansion.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <param name="sinX">Sine of the Angle</param>
		/// <param name="cosX">Cosine of the Angle</param>
		public static void SinCos (int angleInRadians, out decimal sinX, out decimal cosX)
		{
			decimal dX = (decimal) angleInRadians;
			SinCos (dX, out sinX, out cosX);
			return;
		}

		#endregion //SinCos Methods

		#region Tan Methods

		/// <overloads>
		/// <para>Computes the Tangent of the given angle (in Radians) using <see cref="SinCos"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Tangent of the given angle (in Radians) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Tangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal Tan (decimal angleInRadians)
		{
			decimal sinX;
			decimal cosX;
			SinCos (angleInRadians, out sinX, out cosX);

			if (CompareOps.ValueIsZero (cosX))
			{
				throw (new DivideByZeroException());
			}
			return sinX / cosX;
		}

		/// <summary>
		/// Computes the Tangent of the given angle (in Radians) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Tangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal Tan (double angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Tan (dX);
		}

		/// <summary>
		/// Computes the Tangent of the given angle (in Radians) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Tangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal Tan (int angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Tan (dX);
		}

		#endregion //Tan Methods

		#region Asin Methods

		/// <overloads>
		/// <para>Computes the angle (in Radians) that resulted in this Sine Value using Series Expansion.
		/// Thus computing the inverse sine or arcsin.</para>
		/// <para>Methods for decimal and double Parameters. 
		/// Integer Parameters don't make a lot of sense when values are between -1 and 1.</para>
		/// </overloads>
		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Sine Value using Series Expansion.
		/// Thus computing the inverse sine or arcsin.
		/// </summary>
		/// <param name="x">Sine of the Angle</param>
		/// <returns>Angle in Radians in the range of -PI/2 to PI/2 inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Asin (decimal x)
		{

			#region Arguments Check

			if (CompareOps.SameValue (x, 1.0m))
			{
				return DecimalGlobals.PIOver2;
			}

			if (CompareOps.SameValue (x, -1.0m))
			{
				return -DecimalGlobals.PIOver2;
			}

			if (CompareOps.ValueIsZero (x))
			{
				return 0.0m;
			}

			if ((CompareOps.GreaterValue (x, 1.0m) || CompareOps.LesserValue (x, -1.0m)))
			{
				throw (new ArithmeticException());
			}

			#endregion

			const int MaxIterations = 50000;
			decimal Y = x;
			bool Done = false;
			decimal Result = Y;
			decimal OldResult = Result;
			int Cnt = 1;

			do
			{
				Cnt = Cnt + 2;
				Y = (Y / (Cnt - 1)) * (Cnt - 2)* x * x;
				Result = Result + Y / Cnt;

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaxIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Asin Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Sine Value using Series Expansion.
		/// Thus computing the inverse sine or arcsin.
		/// </summary>
		/// <param name="x">Sine of the Angle</param>
		/// <returns>Angle in Radians in the range of -PI/2 to PI/2 inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Asin (double x)
		{
			decimal dX = (decimal) x;
			return Asin (dX);
		}

		#endregion //Asin Methods

		#region Acos Methods

		/// <overloads>
		/// <para>Computes the angle (in Radians) that resulted in this Cosine Value using <see cref="Asin"/>.
		/// Thus computing the inverse cosine or arccos.</para>
		/// <para>Methods for decimal and double Parameters. 
		/// Integer Parameters don't make a lot of sense when values are between -1 and 1.</para>
		/// </overloads>
		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Cosine Value using <see cref="Asin"/>.
		/// Thus computing the inverse cosine or arccos.
		/// </summary>
		/// <param name="x">Cosine of the Angle</param>
		/// <returns>Angle in Radians in the range of 0 to PI inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Acos (decimal x)
		{

			#region Arguments Check

			if (CompareOps.SameValue (x, 1.0m))
			{
				return 0.0m;
			}

			if (CompareOps.SameValue (x, -1.0m))
			{
				return DecimalGlobals.PI;
			}

			if (CompareOps.ValueIsZero (x))
			{
				return DecimalGlobals.PIOver2;
			}

			if ((CompareOps.GreaterValue (x, 1.0m) || CompareOps.LesserValue (x, -1.0m)))
			{
				throw (new ArithmeticException());
			}

			#endregion

			return DecimalGlobals.PIOver2 - Asin (x);
		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Cosine Value using <see cref="Asin"/>.
		/// Thus computing the inverse cosine or arccos.
		/// </summary>
		/// <param name="x">Cosine of the Angle</param>
		/// <returns>Angle in Radians in the range of 0 to PI inclusive.</returns>
		/// <exception cref="ArithmeticException">
		/// Thrown when abs (Value) &gt; 1.
		/// </exception>
		public static decimal Acos (double x)
		{
			decimal dX = (decimal) x;
			return Acos (dX);
		}

		#endregion //Acos Methods

		#region Atan Methods

		private static decimal AtanLT1 (decimal x)
		{
			const int MaxIterations = 50000;
			int Cnt = 1;
			decimal Result = 0.0m;
			decimal Y = x;
			bool Done = false;
			bool Negative = false;
			Result = Y;
			decimal OldResult = Result;

			do
			{
				Negative = !Negative;
				Cnt = Cnt + 2;
				Y = Y  * x * x;

				if (Negative)
				{
					Result = Result - Y / Cnt;
				}
				else
				{
					Result = Result + Y / Cnt;
				}

				if (Result == OldResult)
				{

					Done = true;
				}
				else
				{
					OldResult = Result;
				}
			}

			while ((!Done) && (Cnt < MaxIterations));

			#region Count

#if (SHOWCNT)
			System.Console.WriteLine ("Atan Cnt = " + Cnt.ToString());
#endif

			return Result;

			#endregion //Count

		}

		/// <overloads>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent Value using Series Expansion.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// <para>If you want to cover all quadrants then you may want to use <see cref="Atan2"/></para>
		/// </overloads>
		/// <summary>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent Value using Series Expansion.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>If you want to cover all quadrants then you may want to use <see cref="Atan2"/></para>
		/// </summary>
		/// <param name="x">Tangent of the Angle</param>
		/// <returns>Angle in Radians in the range of -PI/2 to PI/2 inclusive.</returns>
		public static decimal Atan (decimal x)
		{

			#region Arguments Check

			if (CompareOps.SameValue (x, 1.0m))
			{
				return DecimalGlobals.PIOver4;
			}

			if (CompareOps.SameValue (x, -1.0m))
			{
				return -DecimalGlobals.PIOver4;
			}

			if (CompareOps.ValueIsZero (x))
			{
				return 0.0m;
			}

			#endregion

			if (System.Math.Abs (x) < 1.0m)
			{
				return AtanLT1(x);
			}
			else
			{

				if (x > 0.0m)
				{
					return DecimalGlobals.PIOver2 - AtanLT1 (1 / x);
				}
				else
				{
					return -(DecimalGlobals.PIOver2 - AtanLT1 (1 / (-x)));
				}
			}
		}

		/// <summary>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent Value using Series Expansion.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>If you want to cover all quadrants then you may want to use <see cref="Atan2"/></para>
		/// </summary>
		/// <param name="x">Tangent of the Angle</param>
		/// <returns>Angle in Radians in the range of -PI/2 to PI/2 inclusive.</returns>
		public static decimal Atan (double x)
		{
			decimal dX = (decimal) x;
			return Atan (dX);
		}

		/// <summary>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent Value using Series Expansion.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>If you want to cover all quadrants then you may want to use <see cref="Atan2"/></para>
		/// </summary>
		/// <param name="x">Tangent of the Angle</param>
		/// <returns>Angle in Radians in the range of -PI/2 to PI/2 inclusive.</returns>
		public static decimal Atan (int x)
		{
			decimal dX = (decimal) x;
			return Atan (dX);
		}

		#endregion //Atan Methods

		#region Atan2 Methods

		/// <overloads>
		/// <para>Computes the angle (in Radians) that resulted in this Tangent value of y / x using <see cref="Atan"/>.
		/// Thus computing the inverse tangent or arctan.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Tangent value of y / x using <see cref="Atan"/>.
		/// Thus computing the inverse tangent or arctan.
		/// </summary>
		/// <param name="y">Y Co-ordinate</param>
		/// <param name="x">X Co-ordinate</param>
		/// <returns>Angle in Radians in the range of -PI (exclusive) to PI (inclusive).</returns>
		public static decimal Atan2 (decimal y, decimal x)
		{

			#region Arguments Check

			if (CompareOps.ValueIsZero (x))
			{

				if (CompareOps.ValueIsZero (y))
				{
					return 0.0m;
				}

				if (CompareOps.ValueIsPositive (y))
				{
					return DecimalGlobals.PIOver2;
				}
				else
				{
					return -DecimalGlobals.PIOver2;
				}
			}

			if (CompareOps.ValueIsZero (y))
			{

				if (CompareOps.ValueIsPositive (x))
				{
					return 0.0m;
				}
				else
				{
					return DecimalGlobals.PI;
				}
			}

			#endregion

			decimal Z = Atan (System.Math.Abs (y / x));

			if (CompareOps.ValueIsPositive (x))
			{

				if (CompareOps.ValueIsPositive (y))
				{
					return Z;
				}
				else
				{
					return -Z;
				}
			}
			else
			{

				if (CompareOps.ValueIsPositive (y))
				{
					return DecimalGlobals.PI - Z;
				}
				else
				{
					return -DecimalGlobals.PI + Z;
				}
			}
		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Tangent value of y / x using <see cref="Atan"/>.
		/// Thus computing the inverse tangent or arctan.
		/// </summary>
		/// <param name="x">X Co-ordinate</param>
		/// <param name="y">Y Co-ordinate</param>
		/// <returns>Angle in Radians in the range of -PI (exclusive) to PI (inclusive).</returns>
		public static decimal Atan2 (double x, double y)
		{
			decimal dX = (decimal) x;
			decimal dY = (decimal) y;
			return Atan2 (dX, dY);
		}

		/// <summary>
		/// Computes the angle (in Radians) that resulted in this Tangent value of y / x using <see cref="Atan"/>.
		/// Thus computing the inverse tangent or arctan.
		/// </summary>
		/// <param name="x">X Co-ordinate</param>
		/// <param name="y">Y Co-ordinate</param>
		/// <returns>Angle in Radians in the range of -PI (exclusive) to PI (inclusive).</returns>
		public static decimal Atan2 (int x, int y)
		{
			decimal dX = (decimal) x;
			decimal dY = (decimal) y;
			return Atan2 (dX, dY);
		}

		#endregion //Atan2 Methods

		#region  Cosec Methods

		/// <overloads>
		/// <para>Computes the Cosecant of the given angle (in Radians) using <seealso cref="Sin"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Radians) using <seealso cref="Sin"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal Cosec (decimal angleInRadians)
		{
			decimal x = Sin (angleInRadians);
			if (CompareOps.ValueIsZero (x))
			{
				throw new DivideByZeroException ();
			}

			return 1 / x;
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Radians) using <seealso cref="Sin"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal Cosec (double angleInRadians)
		{
			decimal x = Sin (angleInRadians);
			if (CompareOps.ValueIsZero (x))
			{
				throw new DivideByZeroException ();
			}

			return 1 / x;
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Radians) using <seealso cref="Sin"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal Cosec (int angleInRadians)
		{
			decimal x = Sin (angleInRadians);
			if (CompareOps.ValueIsZero (x))
			{
				throw new DivideByZeroException ();
			}

			return 1 / x;
		}

		#endregion //Sec Methods

		#region  Sec Methods

		/// <overloads>
		/// <para>Computes the Secant of the given angle (in Radians) using <seealso cref="Cos"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// <para>Computes the Secant of the given angle (in Radians) using <seealso cref="Cos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal Sec (decimal angleInRadians)
		{
			decimal x = Cos (angleInRadians);
			if (CompareOps.ValueIsZero (x))
			{
				throw new DivideByZeroException ();
			}

			return 1 / x;
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Radians) using <seealso cref="Cos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal Sec (double angleInRadians)
		{
			decimal x = Cos (angleInRadians);
			if (CompareOps.ValueIsZero (x))
			{
				throw new DivideByZeroException ();
			}

			return 1 / x;
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Radians) using <seealso cref="Cos"/>.</para>
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal Sec (int angleInRadians)
		{
			decimal x = Cos (angleInRadians);
			if (CompareOps.ValueIsZero (x))
			{
				throw new DivideByZeroException ();
			}

			return 1 / x;
		}

		#endregion //Sec Methods

		#region Cot Methods

		/// <overloads>
		/// <para>Computes the Cotangent of the given angle (in Radians) using <see cref="SinCos"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Cotangent of the given angle (in Radians) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal Cot (decimal angleInRadians)
		{
			decimal sinX;
			decimal cosX;
			SinCos (angleInRadians, out sinX, out cosX);

			if (CompareOps.ValueIsZero (sinX))
			{
				throw (new DivideByZeroException());
			}
			return cosX / sinX;
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Radians) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal Cot (double angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Cot (dX);
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Radians) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInRadians">Angle in Radians</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal Cot (int angleInRadians)
		{
			decimal dX = (decimal) angleInRadians;
			return Cot (dX);
		}

		#endregion //Cot Methods

		#endregion //Radian Trig Ops

		#region Degree Trig Ops

		#region TanDeg Methods

		/// <overloads>
		/// <para>Computes the Tangent of the Angle, given the Angle in Degrees, using <see cref="Tan"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Tangent of the Angle, given the Angle in Degrees, using <see cref="Tan"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Tangent of the Angle</returns>
		public static decimal TanDeg (decimal angleInDegrees)
		{
			return Tan (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Tangent of the Angle, given the Angle in Degrees, using <see cref="Tan"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Tangent of the Angle</returns>
		public static decimal TanDeg (double angleInDegrees)
		{
			return Tan (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Tangent of the Angle, given the Angle in Degrees, using <see cref="Tan"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Tangent of the Angle</returns>
		public static decimal TanDeg (int angleInDegrees)
		{
			return Tan (Deg2Rad (angleInDegrees));
		}

		#endregion //TanDeg Methods

		#region SinDeg Methods

		/// <overloads>
		/// <para>Computes the Sine of the Angle, given the Angle in Degrees, using <see cref="Sin"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Sine of the Angle, given the Angle in Degrees, using <see cref="Sin"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal SinDeg (decimal angleInDegrees)
		{
			return Sin (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Sine of the Angle, given the Angle in Degrees, using <see cref="Sin"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal SinDeg (double angleInDegrees)
		{
			return Sin (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Sine of the Angle, given the Angle in Degrees, using <see cref="Sin"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Sine of the Angle</returns>
		public static decimal SinDeg (int angleInDegrees)
		{
			return Sin (Deg2Rad (angleInDegrees));
		}

		#endregion //SinDeg Methods

		#region CosDeg Methods

		/// <overloads>
		/// <para>Computes the Cosine of the Angle, given the Angle in Degrees, using <see cref="Cos"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Cosine of the Angle, given the Angle in Degrees, using <see cref="Cos"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal CosDeg (decimal angleInDegrees)
		{
			return Cos (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Cosine of the Angle, given the Angle in Degrees, using <see cref="Cos"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal CosDeg (double angleInDegrees)
		{
			return Cos (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Cosine of the Angle, given the Angle in Degrees, using <see cref="Cos"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cosine of the Angle</returns>
		public static decimal CosDeg (int angleInDegrees)
		{
			return Cos (Deg2Rad (angleInDegrees));
		}

		#endregion //CosDeg Methods

		#region AtanDeg Methods

		/// <overloads>
		/// <para>Computes the Inverse Tangent returning the Angle in Degrees.
		/// Uses <see cref="Atan"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Inverse Tangent returning the Angle in Degrees.
		/// Uses <see cref="Atan"/>.
		/// </summary>
		/// <param name="x">Tangent of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AtanDeg (decimal x)
		{
			return Rad2Deg (Atan (x));
		}

		/// <summary>
		/// Computes the Inverse Tangent returning the Angle in Degrees.
		/// Uses <see cref="Atan"/>.
		/// </summary>
		/// <param name="x">Tangent of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AtanDeg (double x)
		{
			decimal dX = (decimal) x; 
			return AtanDeg (dX);
		}

		/// <summary>
		/// Computes the Inverse Tangent returning the Angle in Degrees.
		/// Uses <see cref="Atan"/>.
		/// </summary>
		/// <param name="x">Tangent of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AtanDeg (int x)
		{
			decimal dX = (decimal) x; 
			return AtanDeg (dX);
		}

		#endregion //AtanDeg Methods

		#region Atan2Deg Methods

		/// <overloads>
		/// <para>Computes the Inverse Tangent, where the Tangent is y /x,  returning the Angle in Degrees.
		/// Uses <see cref="Atan2"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Inverse Tangent, where the Tangent is y /x, returning the Angle in Degrees.
		/// Uses <see cref="Atan2"/>.
		/// </summary>
		/// <param name="y">Y Co-ordinate</param>
		/// <param name="x">X Co-ordinate</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Atan2Deg (decimal y, decimal x)
		{
			return Rad2Deg (Atan2 (y, x));
		}

		/// <summary>
		/// Computes the Inverse Tangent, where the Tangent is y /x, returning the Angle in Degrees.
		/// Uses <see cref="Atan2"/>.
		/// </summary>
		/// <param name="y">Y Co-ordinate</param>
		/// <param name="x">X Co-ordinate</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Atan2Deg (double y, double x)
		{
			decimal dX = (decimal) x; 
			decimal dY = (decimal) y; 
			return Atan2Deg (dY, dX);
		}

		/// <summary>
		/// Computes the Inverse Tangent, where the Tangent is y /x, returning the Angle in Degrees.
		/// Uses <see cref="Atan2"/>.
		/// </summary>
		/// <param name="y">Y Co-ordinate</param>
		/// <param name="x">X Co-ordinate</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal Atan2Deg (int y, int x)
		{
			decimal dX = (decimal) x; 
			decimal dY = (decimal) y; 
			return Atan2Deg (dY, dX);
		}

		#endregion //Atan2Deg Methods

		#region AsinDeg Methods

		/// <overloads>
		/// <para>Computes the Inverse Sine returning the Angle in Degrees.
		/// Uses <see cref="Asin"/>.</para>
		/// <para>Methods for decimal and double Parameters. 
		/// Integer Parameters don't make a lot of sense when values are between -1 and 1.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Inverse Sine returning the Angle in Degrees.
		/// Uses <see cref="Asin"/>.
		/// </summary>
		/// <param name="x">Sine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AsinDeg (decimal x)
		{
			return Rad2Deg (Asin (x));
		}

		/// <summary>
		/// Computes the Inverse Sine returning the Angle in Degrees.
		/// Uses <see cref="Asin"/>.
		/// </summary>
		/// <param name="x">Sine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AsinDeg (double x)
		{
			decimal dX = (decimal) x; 
			return AsinDeg (dX);
		}

		#endregion //AsinDeg Methods

		#region AcosDeg Methods

		/// <overloads>
		/// <para>Computes the Inverse Cosine returning the Angle in Degrees.
		/// Uses <see cref="Acos"/>.</para>
		/// <para>Methods for decimal and double Parameters. 
		/// Integer Parameters don't make a lot of sense when values are between -1 and 1.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Inverse Cosine returning the Angle in Degrees.
		/// Uses <see cref="Acos"/>.
		/// </summary>
		/// <param name="x">Cosine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AcosDeg (decimal x)
		{
			return Rad2Deg (Acos (x));
		}

		/// <summary>
		/// Computes the Inverse Cosine returning the Angle in Degrees.
		/// Uses <see cref="Acos"/>.
		/// </summary>
		/// <param name="x">Cosine of the Angle</param>
		/// <returns>Angle in Degrees</returns>
		public static decimal AcosDeg (double x)
		{
			decimal dX = (decimal) x; 
			return AcosDeg (dX);
		}

		#endregion //AcosDeg Methods


		#region  CosecDeg Methods

		/// <overloads>
		/// <para>Computes the Cosecant of the given angle (in Degrees) using <seealso cref="Sin"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Degrees) using <seealso cref="Sin"/>.</para>
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal CosecDeg (decimal angleInDegrees)
		{
			return Cosec (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Degrees) using <seealso cref="Sin"/>.</para>
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal CosecDeg (double angleInDegrees)
		{
			return Cosec (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Cosecant of the given angle (in Degrees) using <seealso cref="Sin"/>.</para>
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cosecant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal CosecDeg (int angleInDegrees)
		{
			return Cosec (Deg2Rad (angleInDegrees));
		}

		#endregion //CosecDeg Methods

		#region  Sec Methods

		/// <overloads>
		/// <para>Computes the Secant of the given angle (in Degrees) using <seealso cref="Cos"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// <para>Computes the Secant of the given angle (in Degrees) using <seealso cref="Cos"/>.</para>
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal SecDeg (decimal angleInDegrees)
		{
			return Sec (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Degrees) using <seealso cref="Cos"/>.</para>
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal SecDeg (double angleInDegrees)
		{
			return Sec (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// <para>Computes the Secant of the given angle (in Degrees) using <seealso cref="Cos"/>.</para>
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Secant of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the PI/2 or 3*PI/2 Axes.
		/// </exception>
		public static decimal SecDeg (int angleInDegrees)
		{
			return Sec (Deg2Rad (angleInDegrees));
		}

		#endregion //SecDeg Methods

		#region CotDeg Methods

		/// <overloads>
		/// <para>Computes the Cotangent of the given angle (in Degrees) using <see cref="SinCos"/>.</para>
		/// <para>Methods for decimal, double and int Parameters.</para>
		/// </overloads>
		/// <summary>
		/// Computes the Cotangent of the given angle (in Degrees) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal CotDeg (decimal angleInDegrees)
		{
			return Cot (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Degrees) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal CotDeg (double angleInDegrees)
		{
			return Cot (Deg2Rad (angleInDegrees));
		}

		/// <summary>
		/// Computes the Cotangent of the given angle (in Degrees) using <see cref="SinCos"/>.
		/// </summary>
		/// <param name="angleInDegrees">Angle in Degrees</param>
		/// <returns>Cotangent of the Angle</returns>
		/// <exception cref="DivideByZeroException">
		/// Thrown when the angle falls on the 0 or PI Axes.
		/// </exception>
		public static decimal CotDeg (int angleInDegrees)
		{
			return Cot (Deg2Rad (angleInDegrees));
		}

		#endregion //CotDeg Methods


		#endregion //Degree Trig Ops

		#endregion //Trigonometry

	}
}
